BUILDSCRIPTDIR=$(pwd)
export ARCHIVEDIR=$(pwd)
TARGET=arm-linux
PREFIX=/usr/local/devkitPro/devkitGP2X

export BINUTILS_VER=2.16.1
export GCC_VER=4.0.2
export GLIBC_VER=2.3.5
export HEADER_VER=2.6.12.0

export BINUTILS="binutils-$BINUTILS_VER.tar.bz2"
export GCC_CORE="gcc-core-$GCC_VER.tar.bz2"
export GCC_GPP="gcc-g++-$GCC_VER.tar.bz2"
export GLIBC="glibc-$GLIBC_VER.tar.bz2"
export GLIBC_LINUXTHREADS="glibc-linuxthreads-$GLIBC_VER.tar.bz2"
export LINUX_HEADERS="linux-libc-headers-$HEADER_VER.tar.bz2"

BINUTILS_URL="http://ftp.gnu.org/gnu/binutils"
GCC_URL="http://ftp.gnu.org/gnu/gcc/gcc-$GCC_VER"
GLIBC_URL="http://ftp.gnu.org/gnu/glibc"
HEADERS_URL="http://ep09.pld-linux.org/~mmazur/linux-libc-headers"

dopatch() {
	patchlist=$(pwd)/patches/$1/*.patch
	srcdir=$(pwd)/$1

	for arg in $patchlist   
	do
		echo $srcdir $arg
		patch -p1 -d $srcdir -i $arg
	done
}


SELECTION=0

while [ $SELECTION -eq 0 ]
do
  echo
  echo "This script will build and install devkitGP2X. Please select the action you require"
  echo
  echo "1: Download, unpack, patch and build."
  echo "2: Unpack, patch and build."
  echo "3: Build from already patched sources."
  read SELECTION

  if [ "$SELECTION" -ne 1 -a "$SELECTION" -ne 2 -a "$SELECTION" -ne 3 ]
  then
      SELECTION=0
  fi
done


if [ "$SELECTION" -eq 1 ]
then
  wget -c $BINUTILS_URL/$BINUTILS
  wget -c $GCC_URL/$GCC_CORE
  wget -c $GCC_URL/$GCC_GPP
  wget -c $GLIBC_URL/$GLIBC
  wget -c $GLIBC_URL/$GLIBC_LINUXTHREADS
  wget -c $HEADERS_URL/$LINUX_HEADERS
fi

if [ "$SELECTION" -eq 1 -o "$SELECTION" -eq 2 ]
then
  tar -xvjf $ARCHIVEDIR/$BINUTILS
  tar -xvjf $ARCHIVEDIR/$GCC_CORE
  tar -xvjf $ARCHIVEDIR/$GCC_GPP
  tar -xvjf $ARCHIVEDIR/$GLIBC
  tar -xvjf $ARCHIVEDIR/$LINUX_HEADERS

  cd glibc-$GLIBC_VER
  tar -xvjf $ARCHIVEDIR/$GLIBC_LINUXTHREADS

  cd ..

  dopatch binutils-$BINUTILS_VER
  dopatch gcc-$GCC_VER
  dopatch glibc-$GLIBC_VER
fi

SYSROOT=${PREFIX}/sysroot
export PATH=$PATH:${PREFIX}/bin
export ARCH=arm
export CROSS_COMPILE=${TARGET}-
export MAKE=make
 
mkdir -p ${SYSROOT}/usr/include
mkdir -p ${SYSROOT}/usr/lib

cp -a $BUILDSCRIPTDIR/linux-libc-headers-$HEADER_VER/include/asm-arm ${SYSROOT}/usr/include/asm
cp -a $BUILDSCRIPTDIR/linux-libc-headers-$HEADER_VER/include/linux ${SYSROOT}/usr/include/linux
 
mkdir -p build/binutils
cd build/binutils
 
../../binutils-2.16.1/configure --prefix=${PREFIX} --disable-nls --disable-shared --target=${TARGET} --with-sysroot=${SYSROOT} || { echo "Error configuring #binutils"; exit 1; }
 
$MAKE || { echo "Error building binutils"; exit 1; }
$MAKE install || { echo "Error installing binutils"; exit 1; }
 
cd $BUILDSCRIPTDIR
 
mkdir -p build/glibc-$GLIBC_VER-headers
cd build/glibc-$GLIBC_VER-headers
CC=gcc ../../glibc-$GLIBC_VER/configure --prefix=/usr --host=$TARGET --enable-add-ons=linuxthreads --with-headers=${SYSROOT}/usr/include --disable-nls || { echo "Error configuring glibc"; exit 1; }
make cross-compiling=yes install_root=${SYSROOT} install-headers 2>&1 || { echo "Error installing glibc headers"; exit 1; }
touch ${SYSROOT}/usr/include/gnu/stubs.h
cp bits/stdio_lim.h ${SYSROOT}/usr/include/bits/stdio_lim.h
 
cd $BUILDSCRIPTDIR
 
mkdir -p build/gcc_stage1
cd build/gcc_stage1
 
../../gcc-$GCC_VER/configure \
	--prefix=$PREFIX --target=${TARGET} \
	--with-sysroot=$SYSROOT \
	--enable-languages=c --disable-win32-registry --disable-nls  \
	--with-float=soft --with-cpu=arm920t --enable-cxx-flags=-mcpu=arm920t --disable-multilib \
	--disable-shared \
	--enable-threads=no \
	--enable-symvers=gnu \
	--enable-__cxa_atexit \
	--with-newlib || { echo "Error configuring gcc stage1"; exit 1; }

$MAKE all-gcc || { echo "Error building gcc stage1"; exit 1; }
$MAKE install-gcc  || { echo "Error installing gcc stage1"; exit 1; }
 
cd $BUILDSCRIPTDIR
BUILD=$(gcc-$GCC_VER/config.guess)

mkdir -p build/glibc-$GLIBC_VER
cd build/glibc-$GLIBC_VER
BUILD_CC=gcc CC=${CROSS_COMPILE}gcc \
AR=${CROSS_COMPILE}ar \
RANLIB=${CROSS_COMPILE}ranlib \
AS=${CROSS_COMPILE}as \
LD=${CROSS_COMPILE}ld ../../glibc-$GLIBC_VER/configure --prefix=/usr --build=$BUILD --host=$TARGET --target=$TARGET --with-headers=${SYSROOT}/usr/include --without-fp --without-__thread --enable-add-ons=linuxthreads  || { echo "Error configuring glibc"; exit 1; }
$MAKE 2>&1 || { echo "Error building glibc"; exit 1; }
$MAKE install_root=${SYSROOT} install || { echo "Error installing glibc"; exit 1; }
 
cd $BUILDSCRIPTDIR
HOST=$(gcc-$GCC_VER/config.guess)
mkdir -p build/gcc
cd build/gcc
../../gcc-$GCC_VER/configure --prefix=${PREFIX} \
	--host=$HOST --build=$BUILD --target=${TARGET} \
	--enable-languages=c,c++ \
	--disable-libmudflap \
	--disable-libssp \
	--disable-libstdcxx-pch \
	--with-sysroot=${SYSROOT} \
	--with-build-sysroot=${SYSROOT} \
	--enable-shared --disable-win32-registry \
	--enable-threads=posix \
	--enable-symvers=gnu --enable-__cxa_atexit \
	--disable-nls  --with-float=soft \
	--with-cpu=arm920t --enable-cxx-flags=-mcpu=arm920t \
	--enable-symvers=gnu --enable-c99 --enable-long-long \
	--disable-multilib \
	--with-gnu-as --with-gnu-ld \
	|| { echo "Error configuring gcc"; exit 1; }
 
 
$MAKE || { echo "Error building gcc"; exit 1; }
$MAKE install || { echo "Error installing gcc"; exit 1; }
